<?php

namespace Wb4Wp\Helpers;

use Defuse\Crypto\Exception\BadFormatException;
use Defuse\Crypto\Exception\EnvironmentIsBrokenException;

/**
 * Class OptionsHelper
 * Exposes a wrapper around all the options that we register within the plugin.
 *
 * @package WB4WP\Helpers
 *
 * @access  private
 */
final class OptionsHelper
{
    /**
     * Gets the generated unique id for this WP instance, or will generate a new unique id if none is present.
     *
     * @return string
     */
    public static function get_instance_uuid()
    {
        // Do we already have a UUID?
        $instanceUuid = get_option(WB4WP_INSTANCE_UUID_KEY, null);
        if ($instanceUuid === null) {
            // Just generate one and store it
            $instanceUuid = uniqid();
            add_option(WB4WP_INSTANCE_UUID_KEY, $instanceUuid);
        }

        return $instanceUuid;
    }

    /**
     * Gets the generated handshake token that should be used during setup.
     *
     * @return string
     */
    public static function get_handshake_token()
    {
        // Do we already have a UUID?
        $token = get_option(WB4WP_INSTANCE_HANDSHAKE_TOKEN, null);
        $expiration = self::get_handshake_expiration();
        if ($token === null || $expiration === null || $expiration < time()) {
            // No token is known or it expired, generate a new one
            $token = GuidHelper::generate_guid();
            update_option(WB4WP_INSTANCE_HANDSHAKE_TOKEN, $token);
            update_option(WB4WP_INSTANCE_HANDSHAKE_EXPIRATION, time() + 3600);
        }

        return $token;
    }

    /**
     * Gets the expiration time associated with the generated handshake token.
     *
     * @return int|null
     */
    public static function get_handshake_expiration()
    {
        return get_option(WB4WP_INSTANCE_HANDSHAKE_EXPIRATION, null);
    }

    /**
     * Sets the plugin state
     *
     * @param $value -> Must be of value "new, provisioned, onboarded or suspended"
     */
    public static function set_plugin_state($value)
    {
        $states = [
            'new',
            'provisioned',
            'onboarded',
            'suspended',
        ];

        if (! in_array($value, $states)) {
            return;
        }

        update_option(WB4WP_PLUGIN_STATE, $value);
    }

    /**
     * Unlinks all custom metadata
     *
     * @return void
     */
    public static function unlink()
    {
        $meta_options = [
            WB4WP_ENCRYPTION_KEY_KEY,
            WB4WP_INSTANCE_API_KEY_KEY,
            WB4WP_INSTANCE_ID_KEY,
            WB4WP_INSTANCE_UUID_KEY,
            WB4WP_CONNECTED_ACCOUNT_ID,
            WB4WP_SSO_JWT_TOKEN,
	        WB4WP_SITE_ID,
            WB4WP_PLUGIN_STATE,
            WB4WP_ONBOARDING_ID
        ];

        // remove all meta options
        foreach ($meta_options as $meta_option) {
            delete_option($meta_option);
        }
    }

    /**
     * Set a meta value
     *
     * @param $key
     * @param $value
     * @param false $encrypted
     *
     * @return bool
     */
    public static function set($key, $value, $encrypted = false)
    {
        // todo fix the encryption, if provider == bluehost
        $encrypted = false;

        if ($encrypted) {
            try {
                $return_value = EncryptionHelper::update_option($key, $value);
            } catch (BadFormatException $e) {
                $return_value = false;
            } catch (EnvironmentIsBrokenException $e) {
                $return_value = false;
            }
        } else {
            $return_value = update_option($key, $value);
        }

        return $return_value;
    }

    /**
     * Returns an option value by method
     *
     * @param $method
     *
     * @return mixed|null
     */
    public static function get_by_method($method)
    {
        if (!method_exists(self::class, $method)) {
            return null;
        }

        return self::$method();
    }

    /**
     * Gets a single item
     *
     * @param $key
     * @param null $default
     *
     * @return false|mixed|void|null
     */
    public static function get($key, $default = null, $encrypted = false)
    {
        // todo fix the encryption, if provider == bluehost
        $encrypted = false;

        if ( $encrypted ) {
            return EncryptionHelper::get_option($key, $default);    
        }
        return get_option($key, $default);
    }

    /**
     * Removes a single item
     *
     * @param string $key
     *
     * @return bool
     */
    public static function remove($key)
    {
        return delete_option($key);
    }

    /**
     * Collect multiple options using
     * an array of keys
     *
     * A key may consist of either a method name
     * or a meta key
     *
     * @param array $keys
     *
     * @return array|null
     */
    public static function collect($keys = [])
    {
        if (count($keys) > 0) {
            $collection = [];

            foreach ($keys as $key) {
                $collection[$key] = !empty(self::get_by_method($key)) ? self::get_by_method($key) : self::get($key);
            }

            return $collection;
        }

        return null;
    }

    /**
     * Store multiple options using
     * an array of keys and values
     *
     * @param array $keys
     *
     * @return bool
     */
    public static function store($keys = [])
    {
        if (count($keys) > 0) {
            foreach ($keys as $key => $value) {
                if (method_exists(self::class, $key)) {
                    self::$key($value);
                } else {
                    self::set($key, $value);
                }
            }

            return true;
        }

        return false;
    }
}