<?php

namespace Wb4Wp\Managers;

use Exception;
use Wb4Wp\Helpers\CreativeMailHelper;
use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Helpers\OptionsHelper;
use Wb4Wp\Helpers\ProviderHelper;
use Wb4Wp\Wb4Wp;

/**
 * The AdminManager will manage the admin section of the plugin.
 *
 * @package Wb4Wp\Managers
 */
final class AdminManager
{
    public $provider;

    /**
     * AdminManager constructor.
     */
    public function __construct()
    {
        $this->provider = ProviderHelper::get_provider_name();
    }

    /**
     * Will register all the hooks for the admin portion of the plugin.
     */
    public function add_hooks()
    {
        add_action( 'admin_menu', [$this, 'build_menu'] );
        add_action( 'admin_init', [$this, 'register_global_fonts'] );

        global $pagenow;
        $is_new_post_type_page = $pagenow == 'post-new.php' && !empty( $_GET['post_type'] ) && $_GET['post_type'] == 'page';

        if ( $is_new_post_type_page ) {
            add_action( 'admin_init', [$this, 'page_builder_files'] );
        }

        add_action( 'admin_init', [$this, 'apply_window_middleware'] );
        add_filter( 'page_row_actions', [$this, 'filter_post_row_actions'], 11, 2 );
        add_action( 'admin_enqueue_scripts', [$this, 'disable_gutenberg_popup'] );
        add_action( 'admin_head', [$this, 'add_base_styles'] );

        add_action( 'admin_bar_menu', [$this, 'add_edit_with_wb4wp_button_to_admin_bar'], 999 );
    }

    function apply_window_middleware()
    {
        wp_register_script( 'window-middleware-handle', '', ['wp-components']);
        wp_enqueue_script( 'window-middleware-handle' );

        $this->add_inline_js_data_object('window-middleware-handle', [
            'root' => site_url('/'),
            'nonce' => wp_create_nonce('wp_rest'),
            'creativeMailInstalled' => CreativeMailHelper::is_creative_mail_installed(),
            'pluginVersion' => WB4WP_PLUGIN_VERSION,
            'themeVersion' => ThemeManager::get_current_theme_version(),
        ]);
    }

    /**
     * Disables the Gutenberg welcome popup
     */
    function disable_gutenberg_popup()
    {
        wp_add_inline_script('wp-edit-post', 'wp.data.select("core/edit-post").isFeatureActive("welcomeGuide") && wp.data.dispatch("core/edit-post").toggleFeature("welcomeGuide")');
    }

    public function add_inline_js_data_object($handler, $data)
    {
        wp_add_inline_script(
            $handler,
            "window.websiteBuilder = {
                ...(window.websiteBuilder || {}),
                ...".wp_json_encode($data)."
            }",
            'before'
        );
    }

    public function add_scripts_and_styling($block_name, $assets = ['css', 'js'])
    {
        if(empty($block_name)) {
            throw new Exception("No block Name specified!");
        }

        $asset_file = include_once(WB4WP_PLUGIN_DIR . 'build/'.$block_name.'.asset.php');

        if(in_array("css", $assets)) {
            wp_enqueue_style(
                $block_name,
                WB4WP_PLUGIN_URL . 'build/'.$block_name.'.css',
                [],
                $asset_file[VERSION]
            );
        }

        if(in_array("js", $assets)) {
            wp_enqueue_script(
                $block_name,
                WB4WP_PLUGIN_URL . 'build/'.$block_name.'.js',
                $asset_file[DEPENDENCIES],
                $asset_file[VERSION]
            );
        }
    }

    public function filter_post_row_actions($actions, $post)
    {
        if (OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new') !== 'onboarded') {
            return $actions;
        }

        if (get_post_meta($post->ID, '_wp_page_template', true) === 'wb4wp-template.php') {
            $actions['edit_with_wb4wp_'] = sprintf(
                '<a href="%1$s">%2$s</a>',
                admin_url('admin.php?page=wb4wp-editor&wb4wp-post-id=' . $post->ID),
                __('Edit with ' . WB4WP_PLUGIN_NAME, WB4WP_WP_TEXT_DOMAIN)
            );
        }

        return $actions;
    }

    function go_to_page_editor() 
    {
        wp_nonce_field('wp_rest');
        
        $block_name = 'open-wb4wp-editor';

        $this->add_scripts_and_styling($block_name);

        $ping_url = EnvironmentHelper::get_app_gateway_url('wb4wp/v1.0/instance/ping');

        $this->add_inline_js_data_object($block_name, [
            'wpAdminUrl' => admin_url(),
            'pluginState' => OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new'),
            'provider' => $this->provider,
            'providerKey' => ProviderHelper::get_provider(),
            'providerHome' => EnvironmentHelper::get_app_url(),
            'pingUrl' => $ping_url,
            'dashboardUrl' => EnvironmentHelper::get_dashboard_url(),
        ]);
    }

    function go_to_onboarding() 
    { 
        $landing_url = admin_url('admin.php?page=wb4wp-editor');
        $is_onboarded = ProviderHelper::is_bluehost() && OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new') !== 'new';

        if (!$is_onboarded && !ProviderHelper::is_bluehost()) {
            $landing_url = Wb4Wp::get_instance()->get_wb4wp_manager()->get_signup_url();
        }

        $ping_url = EnvironmentHelper::get_app_gateway_url('wb4wp/v1.0/instance/ping');

        $block_name = 'onboarding';

        switch(ProviderHelper::get_provider()) {
            case ProviderHelper::BLUEHOST:
                $domain = 'Bluehost.com';
                break;
            default:
                $domain = 'Websitebuilder.com';
        }

        echo '<div id="website-builder-onboarding"></div>';

        $this->add_scripts_and_styling($block_name);

        $this->add_inline_js_data_object($block_name, [
            'launchUrl' => $landing_url,
            'pingUrl' => $ping_url,
            'provider' => $this->provider,
            'providerDomain' => $domain,
        ]);
    }

    /**
     * Will build the menu for WP-Admin
     */
    public function build_menu()
    {
        $capability = 'manage_options';

        $menu_title = esc_html__(WB4WP_PLUGIN_NAME, WB4WP_WP_TEXT_DOMAIN);
        $icon = 'icon.svg';

        if (ProviderHelper::is_bluehost()) {
            $menu_title = 'Website Builder';
            $icon = 'bluehost-wp-admin.svg';
        }

        // Did the user complete the entire setup?
        if ( OptionsHelper::get( WB4WP_PLUGIN_STATE, 'new') === 'new' && !ProviderHelper::is_bluehost() ) {
            // Show Dashboard
            add_menu_page(
                WB4WP_PLUGIN_NAME,
                $menu_title,
                $capability,
                'wb4wp-editor',
                array( $this, 'go_to_onboarding' ),
                WB4WP_PLUGIN_URL . $icon,
                60
            );
        } else {
            add_menu_page(
                WB4WP_PLUGIN_NAME,
                $menu_title,
                $capability,
                'wb4wp-editor',
                array( $this, 'go_to_page_editor' ),
                WB4WP_PLUGIN_URL . $icon,
                60
            );
        }
    }

    public function register_global_fonts()
    {
        wp_enqueue_style('load-style-1', 'https://fonts.googleapis.com/css?family=Nunito+Sans:400,700');
        wp_enqueue_style('load-style-2', 'https://fonts.googleapis.com/icon?family=Material+Icons');
    }

    /**
     * Registers all assets so that they will be loaded at the admin pages
     */
    public function page_builder_files()
    {
        $block_name = 'new-page-empty-state-page-type-selector';

        $this->add_scripts_and_styling($block_name);

        $this->add_inline_js_data_object($block_name, [
            'provider' => $this->provider,
        ]);
    }

     /**
     * Generates a menu link based
     * on the current plugin dir
     *
     * @param $route
     *
     * @return string
     */
    public function generate_menu_link($route)
    {
        return WB4WP_PLUGIN_DIR_NAME . $route;
    }

    public function add_base_styles()
    {
        global $post;

        if (!empty($post)) {
            $asset_file = WB4WP_PLUGIN_DIR . '/page-assets/assets_' . $post->ID . '.json';

            if (file_exists($asset_file)) {
                $assets = json_decode(file_get_contents($asset_file));

                if (!empty($assets)) {
                    if (!empty($assets->fonts)) {
                        echo "<link rel='stylesheet' href='" . $assets->fonts . "'>";
                    }

                    if (!empty($assets->baseStyle)) {
                        echo "<style type='text/css'>" . $assets->baseStyle . "</style>";
                    }

                    echo "<style type='text/css'>" . file_get_contents(WB4WP_PLUGIN_DIR . '/page-assets/base-style.css') . "</style>";

                    if (!empty($assets->siteModel)) {
                        // Set partnerId to 999 for now. TODO make a useHosting api setting
                        echo "<script>window._isPublished=true;window._site=" . json_encode( $assets->siteModel ) . ";window._site.partnerId = 999;</script>";
                    }

                    if ( !empty( $assets->featureScript ) ) {
                        echo "<script>" . $assets->featureScript . "</script>";
                    }
                }
            }
        }
    }

    public function add_edit_with_wb4wp_button_to_admin_bar( $wp_admin_bar )
    {
        global $post;
        if ( empty( $post ) ) {
            return;
        }

        $post_template = get_post_meta( $post->ID, '_wp_page_template', true );
        if ( $post_template !== 'wb4wp-template.php' ) {
            return;
        }

        $icon = 'icon.svg';
        if ( ProviderHelper::is_bluehost() ) {
            $icon = 'bluehost-wp-admin.svg';
        }

        echo "
            <style>
            #wpadminbar .edit-with-wb4wp .ab-item:before {
                content: '';
                background-image: url('" . WB4WP_PLUGIN_URL . $icon . "') !important;
                width: 1em;
                height: 1em;
                background-repeat: no-repeat;
                top: 50%;
                transform: translateY(-0.5em);
            }
            
            #wpadminbar .edit-with-wb4wp:hover .ab-item:before, #wpadminbar .edit-with-wb4wp .ab-item:focus:before {
                filter: brightness(0) saturate(100%) invert(59%) sepia(47%) saturate(2341%) hue-rotate(151deg) brightness(93%) contrast(102%);
            }
            </style>
        ";

        $wp_admin_bar->add_node( [
            'id' => 'edit_with_wb4wp',
            'title' => sprintf( __( 'Edit with %s', WB4WP_WP_TEXT_DOMAIN ), ProviderHelper::get_provider_name() ),
            'href' => admin_url( 'admin.php?page=wb4wp-editor&wb4wp-post-id=' . $post->ID ),
            'meta' => array(
                'class' => 'edit-with-wb4wp',
            ),
        ] );
    }

}
