<?php

namespace Wb4Wp\Managers;

use Exception;
use Wb4Wp\Wb4Wp;
use Wb4Wp\Helpers\CreativeMailHelper;
use Wb4Wp\Helpers\OptionsHelper;
use Wb4Wp\Helpers\ProviderHelper;
use Wb4Wp\Helpers\RequestHelper;
use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Models\Attachment;
use Wb4Wp\Models\BlogPost;
use WP_Error;
use WP_REST_Response;

/**
 * Class ApiManager
 *
 * @package WB4WP\Managers
 */
final class ApiManager
{
    const API_NAMESPACE = "wb4wp/v1";
    const ROUTE_METHODS = 'methods';
    const ROUTE_PATH = 'path';
    const ROUTE_CALLBACK = 'callback';
    const ROUTE_PERMISSION_CALLBACK = 'permission_callback';
    const ROUTE_REQUIRES_ADMIN = 'requires_admin';
    const ORIGIN = 'Origin';
    const SUCCESS = 'success';
    const HTTP_STATUS = 'status';

    /**
     * Verify that the API KEY is in
     * the header and that it matches
     * our saved API KEY
     *
     * @return WP_Error|bool
     */
    public function validate_api_key()
    {
        //never cache our rest endpoints
        nocache_headers();

        if ( ! array_key_exists( "HTTP_X_API_KEY", $_SERVER ) ) {
            return RequestHelper::throw_error(
                __('Sorry, you are not allowed to do that.', WB4WP_WP_TEXT_DOMAIN),
                __FILE__,
                __LINE__,
                401,
                true
            );
        }

        $key = OptionsHelper::get(WB4WP_INSTANCE_API_KEY_KEY, null, true);
        $apiKey = $_SERVER["HTTP_X_API_KEY"];
        // verify that api key is valid
        if ( $apiKey === $key ) {
            return true;
        }

        return RequestHelper::throw_error(
            __('Sorry, you are not allowed to do that.', WB4WP_WP_TEXT_DOMAIN),
            __FILE__,
            __LINE__,
            401,
            true
        );
    }

    /**
     * Will add all the hooks that are
     * required to setup our plugin API.
     */
    public function add_hooks()
    {
        add_action('rest_api_init', [$this, 'add_rest_endpoints']);
    }

    /**
     * Registers the custom REST endpoints
     */
    public function add_rest_endpoints()
    {
        // Add the endpoint to handle the callback
        $routes = [
            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/local-update",
             *  tags={"Other"},
             *  summary="Starts the auto-update procedure from a local call (from within WP-admin)",
             *  @OA\Response (
             *    response=200,
             *    description="Process is complete.",
             *    @OA\JsonContent (
             *      @OA\Property (
             *         property="success",
             *         type="boolean",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/local-update',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_update_manager(), 'do_update'],
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/do-update",
             *  tags={"Other"},
             *  summary="Starts the auto update procedure",
             *  @OA\Response (
             *    response=200,
             *    description="Process is complete.",
             *    @OA\JsonContent (
             *      @OA\Property (
             *         property="success",
             *         type="boolean",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/do-update',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_update_manager(), 'do_update'],
                self::ROUTE_REQUIRES_ADMIN => false,
                self::ROUTE_PERMISSION_CALLBACK => function () {
                    return $this->validate_callback();
                },
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/token",
             *  tags={"Other"},
             *  summary="Stores the new SSO JWT for Bluehost",
             *  @OA\Response (
             *    response=200,
             *    description="Token is saved",
             *  )
             * )
             */
            [
                self::ROUTE_PATH => '/token',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_provision_manager(), 'update_token'],
                self::ROUTE_REQUIRES_ADMIN => false,
                self::ROUTE_PERMISSION_CALLBACK => function () {
                    return $this->validate_callback();
                },
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/ping",
             *  tags={"Other"},
             *  summary="Public endpoint that can be used to verify the API is up and running",
             *  @OA\Response (
             *    response=200,
             *    description="Endpoint is working",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="pong",
             *        type="boolean",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/ping',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => function () { return new WP_REST_Response(['pong' => true]); },
                self::ROUTE_REQUIRES_ADMIN => false,
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/fetch-stats",
             *  tags={"Other"},
             *  summary="Fetches the latest stats and stores them",
             *  @OA\Response (
             *    response=200,
             *    description="Stats fetched and stored successfully",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *      ),
             *    ),
             *  ),
             *  @OA\Response (
             *    response=500,
             *    description="Failed to fetch the stats",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *        default=false,
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/fetch-stats',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_stats_manager(), 'fetch_stats'],
                self::ROUTE_REQUIRES_ADMIN => false,
                self::ROUTE_PERMISSION_CALLBACK => function () {
                    return $this->validate_callback();
                },
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/callback",
             *  tags={"Onboarding"},
             *  summary="Stores the account information in the wp_options",
             *  @OA\RequestBody (
             *   description="The account information",
             *   required=true,
             *   @OA\JsonContent (
             *    @OA\Property (
             *     property="api_key",
             *     type="string",
             *    ),
             *    @OA\Property (
             *     property="account_id",
             *     type="integer",
             *    ),
             *    @OA\Property (
             *     property="site_id",
             *     type="integer",
             *    ),
             *   ),
             *  ),
             *  @OA\Response (
             *    response=200,
             *    description="Returns true",
             *  ),
             *  @OA\Response (
             *    response=400,
             *    description="Missing account details",
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/callback',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_instance_manager(), 'handle_callback'],
                self::ROUTE_REQUIRES_ADMIN => false,
                self::ROUTE_PERMISSION_CALLBACK => function () {
                    return $this->validate_callback();
                },
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/start_onboarding",
             *  tags={"Onboarding"},
             *  summary="Sends the onboarding data to be stored in Cloudflare",
             *  @OA\Response (
             *    response=200,
             *    description="Returns the onboarding_url",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *      ),
             *      @OA\Property (
             *        property="onboarding_url",
             *        type="string",
             *      ),
             *    ),
             *  ),
             *  @OA\Response (
             *    response=500,
             *    description="Returns the response from the Cloudflare endpoint",
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/start_onboarding',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_provision_manager(), 'handle_start_onboarding_get'],
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/description",
             *  tags={"Publishing"},
             *  summary="Handles the retrieval of the site model",
             *  @OA\Response (
             *    response=200,
             *    description="Returns the site model",
             *    @OA\JsonContent (ref="#/components/schemas/SiteModel"),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/description',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_description_manager(), 'handle_description_get'],
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/wp_posts",
             *  tags={"Blog"},
             *  summary="Returns a paginated list of blog posts",
             *  @OA\Response (
             *    response=200,
             *    description="Returns the array of blog posts",
             *    @OA\JsonContent (
             *      @OA\Items (
             *        type="object",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/wp_posts',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => function ($request) {
                    try {
                        $page = 1;
                        if (property_exists($request, 'page')) {
                            $page = (int)$request['page'];
                        }

                        $posts_per_page = 10;
                        if (property_exists($request, 'posts_per_page')) {
                            $page = (int)$request['posts_per_page'];
                        }

                        $posts = get_posts(
                            array(
                                'posts_per_page' => $posts_per_page,
                                'paged' => $page,
                                'post_type' => 'post',
                            )
                        );

                        $postData = array();
                        foreach ($posts as $post) {
                            array_push($postData, new BlogPost($post));
                        }

                        return new WP_REST_Response($postData, 200);
                    } catch (Exception $ex) { 
                        RaygunManager::get_instance()->exception_handler($ex);
            
                        // rethrow it
                        throw $ex;
                    }
                },
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/description",
             *  tags={"Publishing"},
             *  summary="Handles the storing of the site model",
             *  @OA\Parameter (
             *    name="HTTP_X_API_KEY",
             *    in="header",
             *    required=true,
             *  ),
             *  @OA\RequestBody (
             *   description="The site model",
             *   required=true,
             *   @OA\JsonContent (
             *    @OA\Property (
             *     property="published_pages",
             *     type="object",
             *    ),
             *    @OA\Property (
             *     property="site_description",
             *     type="object",
             *    ),
             *   ),
             *  ),
             *  @OA\Response (
             *    response=200,
             *    description="Returned when saved successfully"
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/description',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_description_manager(), 'handle_description_post'],
                self::ROUTE_REQUIRES_ADMIN => false,
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/images",
             *  tags={"Other"},
             *  summary="Retrieves the WordPress images",
             *  @OA\Response (
             *    response=200,
             *    description="Returns an array of image attachments",
             *    @OA\JsonContent (
             *      @OA\Items (
             *        type="object",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/images',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_REQUIRES_ADMIN => false,
                self::ROUTE_PERMISSION_CALLBACK => function () {
                    return $this->validate_api_key();
                },
                self::ROUTE_CALLBACK => function () {
                    try {
                        $attachmentData = [];
                        $attachments = get_posts([
                            'post_type' => 'attachment',
                            'post_mime_type' => 'image',
                            'post_status' => 'inherit',
                            'posts_per_page' => -1,
                        ]);

                        foreach ($attachments as $attachment) {
                            array_push($attachmentData, new Attachment($attachment));
                        }

                        return new WP_REST_Response($attachmentData, 200);
                    } catch (Exception $ex) { 
                        RaygunManager::get_instance()->exception_handler($ex);
            
                        // rethrow it
                        throw $ex;
                    }
                },
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/provision",
             *  tags={"Onboarding"},
             *  summary="Handles the provisioning of the plugin. Also sets the plugin status to 'provisioned'",
             *  @OA\RequestBody (
             *   required=false,
             *   @OA\JsonContent (),
             *  ),
             *  @OA\Response (
             *    response=500,
             *    description="Returned when the provisioning failed",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *        default=false,
             *      ),
             *      @OA\Property (
             *        property="auth_manager_url",
             *        type="string",
             *      ),
             *    ),
             *  ),
             *  @OA\Response (
             *    response=200,
             *    description="Returned when provisioned successfully",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *      ),
             *      @OA\Property (
             *        property="signup_url",
             *        type="string",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/provision',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => function () {
                    try {
                        // Clear old plugin id to make sure we start fresh
                        delete_option(WB4WP_INSTANCE_UUID_KEY);

                        if (ProviderHelper::get_provider() === 'bluehost') {
                            // Do Bluehost provision logic
                            return ProviderBluehostManager::provision(OptionsHelper::get(WB4WP_JWT, ''));
                        } else {
                            // Prepare WP-Edit provisioning
                            delete_option(WB4WP_INSTANCE_UUID_KEY);

                            return new WP_REST_Response([
                                self::SUCCESS => true,
                                'signup_url' => Wb4Wp::get_instance()->get_wb4wp_manager()->get_signup_url(),
                            ]);
                        }
                    } catch (Exception $ex) { 
                        RaygunManager::get_instance()->exception_handler($ex);
            
                        // rethrow it
                        throw $ex;
                    }
                },
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/sso",
             *  tags={"Onboarding"},
             *  summary="Handles the sso auth of the plugin",
             *  @OA\RequestBody (
             *   required=false,
             *   @OA\JsonContent (),
             *  ),
             *  @OA\Response (
             *    response=500,
             *    description="Returned when the sso failed",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *        default=false,
             *      ),
             *      @OA\Property (
             *        property="auth_manager_url",
             *        type="string",
             *      ),
             *    ),
             *  ),
             *  @OA\Response (
             *    response=200,
             *    description="Returned when the sso authenticated successfully",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="success",
             *        type="boolean",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/sso',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_provision_manager(), 'handle_sso'],
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/plugin-state",
             *  tags={"Onboarding"},
             *  summary="Retrieves the WordPress images",
             *  @OA\Response (
             *    response=200,
             *    description="Returned if the plugin state was retrieved successfully",
             *    @OA\JsonContent (
             *      @OA\Property (
             *        property="plugin_state",
             *        type="string",
             *        enum={"new", "provisioned", "onboarded", "suspended"},
             *      ),
             *      @OA\Property (
             *        property="has_onboarding_id",
             *        type="boolean",
             *      ),
             *      @OA\Property (
             *        property="has_debug_plugin_installed",
             *        type="boolean",
             *      ),
             *      @OA\Property (
             *        property="provider",
             *        type="string",
             *        enum={"bluehost", "websitebuilder"},
             *      ),
             *      @OA\Property (
             *        property="env",
             *        type="string",
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/plugin-state',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => function () {
                    try {
                        $plugins = array_filter(apply_filters('active_plugins', get_option('active_plugins')), function ($plugin) {
                            return strpos($plugin, 'wb4wp-debug-plugin') !== false;
                        });

                        return new WP_REST_Response(
                            [
                                'plugin_state' => OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new'),
                                'has_onboarding_id' => !empty(OptionsHelper::get(WB4WP_ONBOARDING_ID)),
                                'onboarding_id' => OptionsHelper::get(WB4WP_ONBOARDING_ID),
                                'has_debug_plugin_installed' => !empty($plugins),
                                'provider' => ProviderHelper::get_provider(),
                                'env' => EnvironmentHelper::get_environment(),
                            ]
                        );
                    } catch (Exception $ex) { 
                        RaygunManager::get_instance()->exception_handler($ex);
            
                        // rethrow it
                        throw $ex;
                    }
                },
            ],

            /**
             * Swagger GET route
             *
             * @OA\Get (
             *  path="/wb4wp/v1/drafts",
             *  tags={"Other"},
             *  summary="Retrieves the WordPress draft pages",
             *  @OA\Response (
             *    response=200,
             *    description="Returned if the draft pages were retrieved successfully",
             *    @OA\JsonContent (
             *      @OA\Property (
             *         property="pages",
             *         type="array",
             *         @OA\Items (
             *           type="object",
             *         ),
             *      ),
             *    ),
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/drafts',
                self::ROUTE_METHODS => 'GET',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_word_press_manager(), 'get_draft_pages'],
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/publish",
             *  tags={"Onboarding"},
             *  summary="Handles the publishing of the pages",
             *  @OA\RequestBody (
             *   required=true,
             *   description="Requires an array of ID's to publish",
             *   @OA\JsonContent (
             *     @OA\Items (
             *       type="integer",
             *     ),
             *   ),
             *  ),
             *  @OA\Response (
             *    response=200,
             *    description="Returned when the pages are published successfully",
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/publish',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_word_press_manager(), 'publish_pages'],
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/delete_onboarding_id",
             *  tags={"Onboarding"},
             *  summary="Deletes the onboarding ID from WP_Options",
             *  @OA\Response (
             *    response=204,
             *    description="Returned when the onboarding was removed successfully",
             *  ),
             *  @OA\Response (
             *    response=500,
             *    description="Returned when the onboarding was removed unsuccessfully",
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/delete_onboarding_id',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => [Wb4Wp::get_instance()->get_provision_manager(), 'delete_onboarding_id'],
            ],

            /**
             * Swagger POST route
             *
             * @OA\Post (
             *  path="/wb4wp/v1/set_creative_mail_referrer",
             *  tags={"Other"},
             *  summary="Adds a BHWB flag to the CE4WP_REFERRED_BY option",
             *  @OA\Response (
             *    response=204,
             *    description="Returned when CE4WP_REFERRED_BY has been successfully modified or when it was already set",
             *  ),
             * )
             */
            [
                self::ROUTE_PATH => '/set_creative_mail_referrer',
                self::ROUTE_METHODS => 'POST',
                self::ROUTE_CALLBACK => function () {
                    if (CreativeMailHelper::has_bhwb_referrer() === false) {
                        CreativeMailHelper::set_bhwb_referrer();
                    }

                    return new WP_REST_Response(null, 204);
                },
            ],
        ];

        foreach ($routes as $route) {
            $this->register_route($route);
        }
    }

    /**
     * Registers a route to the WP Rest endpoints for this plugin.
     *
     * @param array $route
     */
    private function register_route(array $route)
    {
        // Make sure the route is valid
        $path = $route[self::ROUTE_PATH];
        $methods = $route[self::ROUTE_METHODS];
        $callback = $route[self::ROUTE_CALLBACK];
        $requires_admin = true;

        if (isset($route[self::ROUTE_REQUIRES_ADMIN])) {
            $requires_admin = $route[self::ROUTE_REQUIRES_ADMIN];
        }

        // Make sure we at least have a path
        if (empty($path)) {
            return;
        }

        // Skip the route if we are not in admin mode
        $is_admin = current_user_can('administrator');
        if (!$is_admin && $requires_admin) {
            return;
        }

        // Default arguments for routes
        $arguments = [
            self::ROUTE_METHODS             => $methods,
            self::ROUTE_CALLBACK            => $callback,
            self::ROUTE_PERMISSION_CALLBACK => function () { return true; },
        ];

        // If the route has its own callback, use that
        if (array_key_exists(self::ROUTE_PERMISSION_CALLBACK, $route)) {
            $arguments[self::ROUTE_PERMISSION_CALLBACK] = $route[self::ROUTE_PERMISSION_CALLBACK];
        }

        register_rest_route(self::API_NAMESPACE, $path, $arguments);
    }

    // TODO this needs to be moved to its own file
    public function validate_callback()
    {
        //never cache our rest endpoints
        nocache_headers();

        if (! array_key_exists("HTTP_X_API_KEY", $_SERVER) ) {
            return new WP_Error('rest_forbidden', __('Sorry, you are not allowed to do that.', WB4WP_WP_TEXT_DOMAIN), array( self::HTTP_STATUS => 401 ));
        }

        $apiKey = $_SERVER["HTTP_X_API_KEY"];
        // Verify handshake
        if ($apiKey === OptionsHelper::get_handshake_token() ) {
            return true;
        }

        return new WP_Error('rest_unauthorized', 'Unauthorized', array( self::HTTP_STATUS => 401 ));
    }
}
