<?php

namespace Wb4Wp\Managers;

use Exception;
use WP_Error;
use WP_REST_Response;
use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Helpers\OptionsHelper;
use Wb4Wp\Helpers\ProviderHelper;
use Wb4Wp\Helpers\RequestHelper;

/**
 * The generic provision manager
 *
 * Class ProvisionManager
 *
 * @package Wb4Wp\Managers
 */
final class ProvisionManager
{
    const CONTENT_TYPE = 'Content-Type';
    const APPLICATION_JSON = 'application/json';
    const HEADERS = 'headers';
    const SUCCESS = 'success';
    const BODY = 'body';
    const ERROR = 'error';

    /**
     * Stores a new sso JWT
     *
     * (this functionality is used by the WP-API
     * which does a renewal on the JWT and sends
     * it back to the plugin)
     *
     * @param $request
     * 
     * @throws Exception
     */
    public function update_token($request)
    {
        try {
            $json = $request->get_json_params();

            if (!empty($json['token'])) {
                OptionsHelper::set(WB4WP_SSO_JWT_TOKEN, $json['token'], true);
            }
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Starts the onboarding process
     *
     * @return WP_REST_Response
     * @throws Exception
     */
    public static function handle_start_onboarding_get()
    {
        try {
            $body = [
                'return_url' => admin_url('admin.php?page=wb4wp-editor'),
                'provider' => ProviderHelper::get_provider(),
                'plugin_instantiated' => true,
            ];

            $response = RequestManager::post(
                EnvironmentHelper::get_app_gateway_url('wb4wp/v1.0/onboarding/model'),
                [
                    self::HEADERS => [
                        self::CONTENT_TYPE => self::APPLICATION_JSON,
                    ],
                    self::BODY => $body,
                ]
            );

            if (!is_wp_error($response)) {
                $json = json_decode($response[self::BODY]);

                if (!empty($json->onboarding_url)) {
                    return new WP_REST_Response(
                        [
                            self::SUCCESS => true,
                            'onboarding_url' => $json->onboarding_url,
                        ]
                    );
                }
            } else {
                RequestHelper::throw_error(wp_json_encode($response), __FILE__, __LINE__, 500);
            }

            return new WP_REST_Response(
                [self::SUCCESS => false]
            );
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Authenticate SSO through our API key
     *
     * @return bool|WP_Error|WP_REST_Response
     * @throws Exception
     */
    public static function handle_sso()
    {
        try {
            $deps_return = null;

            $siteId = OptionsHelper::get(WB4WP_SITE_ID);
            if(empty($siteId)){
                $deps_return = RequestHelper::throw_error(
                    [self::SUCCESS => false, self::ERROR => 'site id empty'],
                    __FILE__,
                    __LINE__,
                    500
                );
            }
            $apiKey = OptionsHelper::get(WB4WP_INSTANCE_API_KEY_KEY, null, true);
            if(empty($apiKey)){
                $deps_return = RequestHelper::throw_error(
                    [self::SUCCESS => false, self::ERROR => 'api key empty'],
                    __FILE__,
                    __LINE__,
                    500
                );
            }
            $connectedAccount = OptionsHelper::get(WB4WP_CONNECTED_ACCOUNT_ID);
            if(empty($connectedAccount)){
                $deps_return = RequestHelper::throw_error(
                    [self::SUCCESS => false, self::ERROR => 'connected account empty'],
                    __FILE__,
                    __LINE__,
                    500
                );
            }
            $get_instance_uuid = OptionsHelper::get_instance_uuid();
            if(empty($connectedAccount)){
                $deps_return = RequestHelper::throw_error(
                    [self::SUCCESS => false, self::ERROR => 'instance uuid empty'],
                    __FILE__,
                    __LINE__,
                    500
                );
            }

            if (!empty($deps_return)) {
                return $deps_return;
            }

            $postBody = [
                'plugin_id' => $get_instance_uuid,
                'instance_url' => get_bloginfo('wpurl'),
                'wordpress_version' => get_bloginfo('version'),
                'plugin_version' => env('WB4WP_PLUGIN_VERSION'),
                'token' => OptionsHelper::get(WB4WP_JWT), // The JWT can be optional, no empty check required
            ];

            $response = RequestManager::post(
                EnvironmentHelper::get_app_gateway_url('wordpress/v1.0/builder/sso'),
                [
                    self::HEADERS => [
                        self::CONTENT_TYPE => self::APPLICATION_JSON,
                        'X-API-KEY' => $apiKey,
                        'X-ACCOUNT-ID' => $connectedAccount,
                    ],
                    self::BODY => $postBody,
                ]
            );

            if (!is_wp_error($response)) {
                $json = json_decode($response[self::BODY]);

                if (!empty($json->authentication)) {
                    // Store the sso jwt token and ttl in the settings
                    OptionsHelper::set(WB4WP_SSO_JWT_TOKEN, $json->authentication, true);
                    OptionsHelper::set(WB4WP_SSO_JWT_TTL, strtotime("+60 minutes", time()));

                    return new WP_REST_Response([
                        self::SUCCESS => true,
                        'jwt' => $json->authentication,
                        'site_id' => $siteId,
                        'plugin_state' => OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new'),
                        'has_onboarding_id' => !empty(OptionsHelper::get(WB4WP_ONBOARDING_ID)),
                        'onboarding_id' => OptionsHelper::get(WB4WP_ONBOARDING_ID),
                    ]);
                }
            }

            return RequestHelper::throw_error(wp_json_encode($response), __FILE__, __LINE__, 500);
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Deletes the onboarding_id from WP_Options
     * 
     * @throws Exception
     */
    public function delete_onboarding_id()
    {
        try {
            if (!OptionsHelper::remove(WB4WP_ONBOARDING_ID)) {
                return new WP_REST_Response('Could not delete onboarding ID', 500);
            }

            return new WP_REST_Response(null, 204);
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

}