<?php

namespace Wb4Wp\Managers;
use Exception;
use Raygun4php\RaygunClient;

/**
 * The RaygunManager will manage the admin section of the plugin.
 *
 * @package Wb4Wp\Managers
 */
final class RaygunManager
{
    // Lets make this a singleton
    private static $instance;
    private $raygun_client;

    /**
     * RaygunManager constructor.
     */
    public function __construct()
    {
        $this->raygun_client = new RaygunClient(env('WB4WP_RAYGUN_PHP_KEY'));
    }
    
    /**
     * Transmits an error to the Raygun.io API
     *
     * @param int    $err_no          The error number
     * @param string $err_str         The error string
     * @param string $err_file        The file the error occurred in
     * @param int    $err_line        The line the error occurred on
     */
    function error_handler($err_no, $err_str, $err_file, $err_line)
    {
        $this->raygun_client->SendError($err_no, $err_str, $err_file, $err_line, self::build_tags(), self::build_custom_user_data());
    }

    /**
     * Transmits an exception to the Raygun.io API
     *
     * @param \Exception $exception      An exception object to transmit
     */
    function exception_handler($exception)
    {
        $this->raygun_client->SendException($exception, self::build_tags(), self::build_custom_user_data());
    }

    function build_tags()
    {
        $tags = [];

        try {
            // Get as many meta data as possible
            $tags['WB4WP_PLUGIN_VERSION'] = env('WB4WP_PLUGIN_VERSION');
            $tags['WB4WP_ENVIRONMENT'] = env('WB4WP_ENVIRONMENT');
            $tags['WB4WP_PROVIDER'] = env('WB4WP_PROVIDER');
        } catch (Exception $e) {
            // do nothing, otherwise we might have an endless loop
        }

        return $tags;
    }

    function build_custom_user_data()
    {
        $userData = [];

        try {
            // Get as many meta data as possible
            $userData['WB4WP_THEME_VERSION'] = env('WB4WP_THEME_VERSION');
            $userData['WB4WP_APP_URL'] = env('WB4WP_APP_URL');
            $userData['WB4WP_APP_GATEWAY_URL'] = env('WB4WP_APP_GATEWAY_URL');
            $userData['WB4WP_APP_DASHBOARD_URL'] = env('WB4WP_APP_DASHBOARD_URL');

            // user data that helps us identify the error
            $userData['wb4wp_connected_account_id'] = get_option('wb4wp_connected_account_id');
            $userData['wb4wp_instance_uuid'] = get_option('wb4wp_instance_uuid');
            $userData['wb4wp_site_id'] = get_option('wb4wp_site_id');
            $userData['wb4wp_jwt_token'] = get_option('wb4wp_jwt_token');
            
        } catch (Exception $e) {
            // do nothing, otherwise we might have an endless loop
        }

        return $userData;
    }

    public static function get_instance()
    {
        if (self::$instance === null) {
            self::$instance = new RaygunManager();
        }

        return self::$instance;
    }
}
