<?php

namespace Wb4Wp\Managers;

use Wb4Wp\Wb4Wp;

/**
 * The TemplateManager will manage the templates of the website.
 *
 * @package Wb4Wp\Managers
 */
final class TemplateManager
{
    // SonarQube dictionary
    const THE_CONTENT = 'the_content';
    const WB4WP_STYLES = 'ee_styles';
    const WB4WP_SCRIPTS = 'ee_scripts';

    /**
     * The array of templates that this plugin tracks.
     *
     * @var array
     */
    protected $templates = [];

    function __construct()
    {
        global $wp_version;

        if (version_compare($wp_version, '5.0.0') >= 0) {
            $this->register_blocks();
        }
    }

    /**
     * Register Gutenberg blocks.
     */
    public function register_blocks()
    {
        register_block_type(
            'wb4wp/block-generic-wb4wp-content',
            [
                'editor_style'  => 'block-generic-wb4wp-content',
                'editor_script' => 'block-generic-wb4wp-content',
            ]
        );
    }

    /**
     * Will add all the hooks that are required to setup our plugin templates.
     */
    public function add_hooks()
    {
        add_filter('theme_page_templates', [$this, 'add_new_template']);

        // Add a filter to the save post to inject out template into the page cache
        add_filter('wp_insert_post_data', [$this, 'register_project_templates']);

        // Add a filter to the template include to determine if the page has our
        // template assigned and return it's path
        add_filter( 'template_include', [$this, 'view_project_template'] );

        // Add your templates to this array.
        $this->templates = [
            'wb4wp-template.php' => 'Website Builder Template',
        ];

        //Register hook to load styles
        add_action( 'wp_enqueue_scripts', [$this, 'ee_enqueue_styles'] );

        add_action( 'template_redirect', [$this, 'thisismyurl_change_theme_manually'] );

        add_action( 'init', [$this, 'expand_allowed_tags'] );
    }

    function expand_allowed_tags()
    {
        global $allowedposttags;

        if ( empty( $allowedposttags['style'] ) ) {
            $allowedposttags['style'] = [
                'scoped' => true
            ];
        }
    }

    function thisismyurl_change_theme_manually()
    {
        if ( 'twentyeleven' !== get_stylesheet() && is_admin() ) {
            switch_theme( 'twentyeleven', 'style.css' );
        }
    }

    function ee_enqueue_styles()
    {
        global $post;
        if ($post !== null && Wb4Wp::get_instance()->get_description_manager()->is_built_with_wb4wp_($post->ID)) {
            $styles = get_post_meta($post->ID, 'WB4WP_PAGE_STYLES', true);
            if ($styles !== null) {
                wp_register_style(self::WB4WP_STYLES, false);
                wp_enqueue_style(self::WB4WP_STYLES);
                wp_add_inline_style(self::WB4WP_STYLES, base64_decode($styles));
            }
        } else {
            $styles = get_option('WB4WP_GLOBAL_STYLES', null);
            if ($styles !== null) {
                wp_register_style(self::WB4WP_STYLES, false);
                wp_enqueue_style(self::WB4WP_STYLES);
                wp_add_inline_style(self::WB4WP_STYLES, base64_decode($styles));
            }
        }
    }

    function ee_enqueue_scripts()
    {
        global $post;
        if (Wb4Wp::get_instance()->get_description_manager()->is_built_with_wb4wp_($post->ID)) {
            $scripts = get_post_meta($post->ID, 'WB4WP_PAGE_SCRIPTS', true);
            if ($scripts !== null) {
                wp_register_style(self::WB4WP_SCRIPTS, false);
                wp_enqueue_style(self::WB4WP_SCRIPTS);
                wp_add_inline_style(self::WB4WP_SCRIPTS, base64_decode($scripts));
            }
        }
    }

    /**
     * Adds our template to the page dropdown for v4.7+
     *
     * @param $posts_templates
     *
     * @return array
     */
    public function add_new_template($posts_templates)
    {
        return array_merge($posts_templates, $this->templates);
    }

    /**
     * Adds our template to the pages cache in order to trick WordPress
     * into thinking the template file exists where it doesn't really exist.
     *
     * @param $atts
     *
     * @return mixed
     */
    public function register_project_templates($atts)
    {
        // Create the key used for the themes cache
        $cache_key = 'page_templates-' . md5(get_theme_root() . '/' . get_stylesheet());

        // Retrieve the cache list.
        // If it doesn't exist, or it's empty prepare an array
        $page_templates = wp_get_theme()->get_page_templates() ?: [];

        // New cache, therefore remove the old one
        wp_cache_delete($cache_key, 'themes');

        // Now add our template to the list of templates by merging our templates
        // with the existing templates array from the cache.
        $page_templates = array_merge($page_templates, $this->templates);

        // Add the modified cache to allow WordPress to pick it up for listing
        // available templates
        wp_cache_add($cache_key, $page_templates, 'themes', 1800);

        return $atts;
    }

    /**
     * Checks if the template is assigned to the page
     *
     * @param $template
     *
     * @return string
     */
    public function view_project_template($template)
    {
        // Get global post
        global $post;

        if ($post) {
            $post_meta = get_post_meta($post->ID, '_wp_page_template', true);

            if (isset($this->templates[$post_meta])) {
                $file = WB4WP_PLUGIN_DIR . 'src/templates/' . $post_meta;

                // Just to be safe, we check if the file exist first
                if (file_exists($file)) {
                    return $file;
                }
            }
        }

        /**
         * Return template if post is empty
         * or if we don't have a custom one defined
         */
        return $template;
    }
}