<?php

namespace Wb4Wp\Managers;

use Exception;
use Wb4Wp\Constants\ThemeConstants;
use ZipArchive;

/**
 * Class ThemeManager
 *
 * @package WB4WP\Managers
 */
final class ThemeManager
{
    public function add_hooks()
    {
        add_action( WB4WP_THEME_UPDATE_CRON, array( $this, 'run_update_cron' ) );
        
        if ( ! wp_next_scheduled( WB4WP_THEME_UPDATE_CRON ) ) {
            // TODO: Revert back to daily
            // $time = ( time() + ( 3600 * mt_rand( 0, 24 ) ) );
            // $recurrence = 'daily';

            // if (EnvironmentHelper::is_test_environment()) {
            $time = ( time() + 900 );
            $recurrence = 'every_15_minutes';
            // }

            wp_schedule_event( $time, $recurrence, WB4WP_THEME_UPDATE_CRON );
        }
    }

    /**
     * Handler for the cron job
     * 
     * @throws Exception
     */
    public function run_update_cron()
    {
        try {
            self::install_theme();
        } catch ( Exception $ex ) {
            RaygunManager::get_instance()->exception_handler( $ex );

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Retrieves the most recent theme version
     *
     * @return string
     */
    public static function get_most_recent_theme()
    {
        $json = wp_remote_get( env( 'WB4WP_THEME_VERSION' ) );

        if ( is_wp_error( $json ) ) {
            return null;
        }

        return json_decode( $json['body'] );
    }

    /**
     * Installs and enables the
     * latest wb4wp theme
     */
    public static function install_theme()
    {
        // 1) Get Info from the latest theme out there
        $new_theme = self::get_most_recent_theme();

        // 2) Make sure that we got a valid response from our server
        if ( empty( $new_theme->version ) ) {
            return null;
        }

        // 3 Check if our theme exists in the theme directory
        $our_theme_is_installed = false;
        $installed_themes = wp_get_themes();

        foreach ( $installed_themes as $theme ) {
            // Find our theme
            if ( strpos( $theme->stylesheet, ThemeConstants::THEME_NAME ) !== false ) {
                // make the fact that we already have the theme installed
                $our_theme_is_installed = true;

                // 0) Only update the theme if we are not on the local docker where we linked the folder of the theme.
                $bitbucket_pipelines_file = get_theme_root() . '/' . $theme->stylesheet . '/bitbucket-pipelines.yml';
                if ( file_exists( $bitbucket_pipelines_file ) ) {
                    continue;
                }

                $exploded_version = explode( '.', $theme->get( 'Version' ) );
                $build = end( $exploded_version );

                if ( count( $exploded_version ) < 2 || intval( $build ) === 0 ) {
                    continue;
                }

                // If the current versions of our theme not == the latest version, we should trigger an update
                if ( $build !== $new_theme->version ) {
                    // The latest version is a different version so we must update. (might be roll back or roll forward)
                    $theme_folder_destination = self::install_the_version_of_our_theme( $new_theme );

                    if ( $theme_folder_destination ) {
                        self::activate_theme( self::get_new_theme_name( $new_theme ) . '/style.css' );
                        $old_theme_directory = trailingslashit( get_theme_root() . '/' . $theme->stylesheet );

                        if ( self::check_if_theme_switched_out( $old_theme_directory ) ) {
                            // Add Extra check to make sure its not deleting the newly created theme (this code triggers if we have a bug in the update system.)
                            if (
                                $old_theme_directory !== $theme_folder_destination
                                && is_dir( $old_theme_directory )
                            ) {
                                self::del_tree( $old_theme_directory );
                            }
                        }
                    }
                } else if ( wp_get_theme()->stylesheet !== $theme->stylesheet ) {
                    self::activate_theme( $theme->stylesheet );
                }
            }
        }

        if ( $our_theme_is_installed === false ) {
            if ( self::install_the_version_of_our_theme( $new_theme ) ) {
                self::activate_theme( self::get_new_theme_name( $new_theme ) . '/style.css' );
            }
        }
    }

    /**
     * Check if the theme has been switched out
     *
     * @param $old_stylesheet
     *
     * @return bool
     */
    private static function check_if_theme_switched_out( $old_stylesheet )
    {
        $stylesheet = get_option( 'theme_switched' );

        if ( $stylesheet && $stylesheet === $old_stylesheet ) {
            $old_theme = wp_get_theme( $stylesheet );

            if ( $old_theme->exists() ) {
                return false;
            }

        }

        return true;
    }

    private static function install_the_version_of_our_theme( $new_theme )
    {
        // 4 Let's to update to the new theme
        // 4 a) Download the new zip, store in a dist folder
        // 4 b) Unzip the new zip
        // 4 c) activate the new theme
        // 4 d) remove the old theme.

        $zip_destination = get_theme_root() . '/wb4wp-wordpress-theme.zip';

        // Remove the file from the previous download
        if ( file_exists( $zip_destination ) ) {
            unlink( $zip_destination );
        }

        // 4a)
        $copy = copy(
            $new_theme->url,
            $zip_destination
        );

        if ( ! $copy ) {
            return false;
        }

        $new_theme_name = self::get_new_theme_name( $new_theme ) . '_' . time();
        $theme_folder_destination = trailingslashit( get_theme_root() . '/' . $new_theme_name );

        // 4b)
        if ( file_exists( $theme_folder_destination ) && is_dir( $theme_folder_destination ) ) {
            self::del_tree( $theme_folder_destination );
        }                        

        $zip = new ZipArchive();
        if ( $zip->open( $zip_destination ) === TRUE ) {
            $zip->extractTo( $theme_folder_destination );
            $zip->close();
        }

        // 5) Clean up zip we just downloaded
        if ( file_exists( $zip_destination ) ) {
            unlink( $zip_destination );
        } 

        // return the folder of the new theme
        return $theme_folder_destination;
    }

    /**
     * Activate the new theme
     *
     * @param $new_theme_name
     */
    private static function activate_theme( $new_theme_name )
    {
        switch_theme( $new_theme_name );
    }

    /**
     * Get the new theme name
     *
     * @param $new_theme
     *
     * @return string
     */
    public static function get_new_theme_name( $new_theme )
    {
        return 'wb4wp-wordpress-theme-' . str_replace( '.', '-', $new_theme->version );
    }

    /**
     * Removes a directory and
     * its contents
     *
     * @param $dir
     *
     * @return bool
     */
    private static function del_tree( $dir )
    {
        $files = array_diff( scandir( $dir ), array( '.', '..' ) );
        foreach ( $files as $file ) {
            ( is_dir( "$dir/$file" ) ) ? self::del_tree( "$dir/$file" ) : unlink( "$dir/$file" );
        }
        return rmdir( $dir );
    }

    /**
     * Returns the version of the current theme
     * if the active theme is ours
     *
     * @return array|false|string|null
     */
    public static function get_current_theme_version()
    {
        $current_theme = wp_get_theme();

        if ( strpos( $current_theme->stylesheet, ThemeConstants::THEME_NAME ) !== false ) {
            return $current_theme->get('Version');
        }

        return null;
    }
}
