<?php

namespace Wb4Wp\Managers;

use Exception;
use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Helpers\ProviderHelper;
use WP_Error;
use WP_REST_Response;

/**
 * Class UpdateManager
 *
 * @package Wb4Wp\Managers
 */
final class UpdateManager
{
    private $config = [
        'wb4wp_update_plugin_file' => 'wb4wp-update-plugin.php',
    ];

    public function __construct()
    {
        $environment = EnvironmentHelper::get_environment();
        $this->config['update_plugin_zip_url'] = 'https://bhb-update.wordpress.wzdev.co/download.php?provider=' . ProviderHelper::get_provider() . '&env=' . $environment;
    }

    public function add_hooks()
    {
        if ( file_exists( ABSPATH . '.maintenance' ) ) {
            add_action( 'admin_head', [$this, 'wp_maintenance_mode'] );
        }

        add_action( WB4WP_PLUGIN_UPDATE_CRON, array( $this, 'do_update' ) );

        // if (EnvironmentHelper::is_test_environment()) {
        add_filter( 'cron_schedules', function( $schedules ) {
            $schedules['every_15_minutes'] = array(
                'interval' => 900,
                'display' => __( 'Once every 15 minutes', WB4WP_WP_TEXT_DOMAIN ),
            );
            return $schedules;
        } );
        // }

        if ( ! wp_next_scheduled( WB4WP_PLUGIN_UPDATE_CRON ) ) {
            // TODO: Revert back to daily
            // $time = ( time() + ( 3600 * mt_rand( 0, 24 ) ) );
            // $recurrence = 'daily';

            // if (EnvironmentHelper::is_test_environment()) {
            $time = ( time() + 900 );
            $recurrence = 'every_15_minutes';
            // }

            wp_schedule_event( $time, $recurrence, WB4WP_PLUGIN_UPDATE_CRON );
        }

        $this->check_for_fs_method();
    }

    /**
     * Checks if the wp-config contains the FS_METHOD
     * If not, this will add it
     */
    private function check_for_fs_method()
    {
        $wp_config = file_get_contents(ABSPATH . 'wp-config.php');

        if (strpos($wp_config, 'FS_METHOD') === false) {
            $wp_config .= PHP_EOL . "/** Auto-generated by " . ProviderHelper::get_provider_name() . " */" . PHP_EOL;
            $wp_config .= "define('FS_METHOD', 'direct');" . PHP_EOL;

            file_put_contents(ABSPATH . 'wp-config.php', $wp_config);
        }
    }

    // Activate WordPress Maintenance Mode
    public function wp_maintenance_mode()
    {
        $plugin_name = WB4WP_PLUGIN_NAME;
        wp_die( "<h1>Under Maintenance</h1><br />We're currently updating the {$plugin_name} plugin. Please hang tight." );
    }

    /**
     * Runs the updater
     *
     * @return WP_REST_Response|WP_Error
     *
     * @throws Exception
     */
    public function do_update()
    {
        try {
            if ( ! $this->should_do_update() ) {
                return new WP_REST_Response( ['success' => true, 'status' => 'update-unavailable'], 200 );
            }

            if ( $this->fs_enabled() ) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
                require_once ABSPATH . 'wp-admin/includes/plugin.php';

                $creds = request_filesystem_credentials(
                    site_url() . '/wp-admin/',
                    '',
                    false,
                    false,
                    []
                );
                if ( ! WP_Filesystem( $creds ) ) {
                    // any problems and we exit
                    wp_die( 'Filesystem Problem' );
                }

                $temp_path = download_url( $this->config['update_plugin_zip_url'] );

                if ( is_wp_error( $temp_path ) ) {
                    wp_die(
                        sprintf(
                            __( 'Error Downloading: <a href="%1$s">%1$s</a> - Error: %2$s', WB4WP_WP_TEXT_DOMAIN ),
                            $this->config['update_plugin_zip_url'],
                            $temp_path->get_error_message()
                        )
                    );
                }

                $result = unzip_file( $temp_path, WP_PLUGIN_DIR );

                if ( is_wp_error( $result ) ) {
                    wp_die(
                        sprintf(
                            __( 'Error Unzipping File: Error: %1$s', WB4WP_WP_TEXT_DOMAIN ),
                            $result->get_error_message()
                        )
                    );
                }

                $update_plugin_path = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . $this->config['wb4wp_update_plugin_file'];

                if ( ! file_exists( $update_plugin_path ) ) {
                    wp_die(
                        __( 'Error Installing Update Plugin.', WB4WP_WP_TEXT_DOMAIN )
                    );
                }

                $activate = activate_plugin( $this->config['wb4wp_update_plugin_file'] );

                if ( is_wp_error( $activate ) ) {
                    wp_die(
                        sprintf(
                            __( 'Error Activating Plugin: Error: %1$s', WB4WP_WP_TEXT_DOMAIN ),
                            $activate->get_error_message()
                        )
                    );
                }

                require_once $update_plugin_path;

                if ( class_exists( 'Wb4WpUpdate' ) ) {
                    \Wb4WpUpdate::instantiate();

                    if ( has_action( 'wb4wp_do_update' ) ) {
                        do_action( 'wb4wp_do_update' );

                        return new WP_REST_Response( ['success' => true, 'status' => 'update-complete'], 200 );
                    }
                }
            }

            return new WP_Error( 'rest_error', 'Update failed', ['status' => 500] );
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Check if the required
     * FS_METHOD is set
     *
     * @return bool
     */
    private function fs_enabled()
    {
        return defined( 'FS_METHOD' ) && FS_METHOD === 'direct';
    }

    /**
     * Check if the {BUILD} text
     * is not in the version of the plugin
     *
     * @return bool
     */
    private function should_do_update()
    {
        $plugin_data = get_file_data(WB4WP_PLUGIN_FILE, array('Version' => 'Version'), false);
        return strpos($plugin_data['Version'], '{BUILD}') === false;
    }
}