<?php

namespace Wb4Wp\Managers;

use DateTime;
use Exception;
use Wb4Wp\Helpers\ProviderHelper;
use WP_Post;
use WP_REST_Response;

/**
 * Class WordPressManager
 *
 * @package WB4WP\Managers
 */
final class WordPressManager
{
    /**
     * Removes all current pages in the WordPress instance.
     */
    public static function reset_wp_instance()
    {
        $pages = get_pages(
            [
                'post_status' => [
                    'publish',
                    'pending',
                    'draft',
                    'auto-draft',
                    'future',
                    'private',
                    'inherit',
                ]
            ]
        );

        foreach ($pages as $page) {
            wp_delete_post($page->ID);
        }
    }

    public function get_draft_pages()
    {
        try {
            $template = 'wb4wp-template.php';
            $drafts = get_pages(['post_status' => ['draft'], 'meta_key' => '_wp_page_template', 'meta_value' => $template]);
            $ee_page_ids = array_column(
                get_pages([
                    'post_type' => 'page',
                    'meta_query' => array(
                        'template_clause' => array(
                            'key' => '_wp_page_template',
                            'value' => $template,
                            'compare' => '=',
                        ),
                        'wb4wp_clause' => array(
                            'key' => 'WB4WP_EDIT_MODE',
                            'compare' => 'EXISTS',
                        ),
                    ),
                ]),
            'ID');
            $ee_revisions = [];

            foreach($ee_page_ids as $ee_page_id) {
                $revision = $this->get_latest_revision($ee_page_id);
                if (!empty($revision)) {
                    $ee_revisions[] = $revision;
                }
            }

            return new WP_REST_Response(array_merge($ee_revisions, $drafts), 200);
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    public function publish_pages($request)
    {
        try {
            foreach ($request->get_json_params() as $pageId) {
                $post = get_post($pageId);

                if (!empty($post)) {
                    if ($post->post_type === 'revision') {
                        $this->restore_post($post);
                        $draft_sections = get_post_meta($post->post_parent, 'WB4WP_DRAFT_SECTIONS', true);

                        if (!empty($draft_sections)) {
                            update_post_meta($post->post_parent, 'WB4WP_PAGE_SECTIONS', $draft_sections);
                            delete_post_meta($post->post_parent, 'WB4WP_DRAFT_SECTIONS');
                        }

                        delete_post_meta($post->post_parent, 'WB4WP_DRAFT');
                        $older_revisions = wp_get_post_revisions($post->post_parent);

                        // After we restore the latest revision, we delete all the old ones.
                        foreach($older_revisions as $revision) {
                            wp_delete_post_revision($revision->ID);
                            delete_post_meta($revision->post_parent, 'WB4WP_DRAFT');
                        }
                    } else {
                        wp_publish_post($pageId);
                    }
                }
            }

            ThemeManager::install_theme();

            if (ProviderHelper::is_bluehost()) {
                update_option('mm_coming_soon', 'false');
            }

            return new WP_REST_Response(null, 204);
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    public function restore_post($post)
    {
        $new_post = get_post($post->post_parent);
        $new_post->post_content = $post->post_content;

        remove_filter('content_save_pre', 'wp_filter_post_kses');
        remove_filter('content_filtered_save_pre', 'wp_filter_post_kses');

        $response = wp_insert_post($new_post);

        add_filter('content_save_pre', 'wp_filter_post_kses');
        add_filter('content_filtered_save_pre', 'wp_filter_post_kses');

        return $response;
    }

    /**
     * Stores a post object as auto-save
     *
     * @param array $post_data
     *
     * @return bool
     */
    public static function store_auto_save(array $post_data)
    {
        $the_future = new DateTime('+20 seconds');
        $post_data = self::_wp_get_allowed_post_data($post_data);
        $post = (array) get_post($post_data['ID']);
        $post_data = array_merge($post, $post_data);
        $post_data['post_modified'] = $the_future->format('Y-m-d H:i:s');
        $post_data['post_modified_gmt'] = gmdate('Y-m-d H:i:s', $the_future->getTimestamp());
        $old_auto_save = wp_get_post_autosave($post_data['ID']);

        if ($old_auto_save) {
            $new_auto_save = _wp_post_revision_data($post_data, true);
            $new_auto_save['ID'] = $old_auto_save->ID;

            // If the new auto-save has the same content as the post, delete the auto-save.
            $auto_save_is_different = false;
            foreach (array_intersect(array_keys($new_auto_save), array_keys(_wp_post_revision_fields($post))) as $field) {
                if (normalize_whitespace($new_auto_save[$field]) !== normalize_whitespace($post[$field])) {
                    $auto_save_is_different = true;
                    break;
                }
            }

            if (!$auto_save_is_different) {
                delete_post_meta($old_auto_save->ID, 'WB4WP_DRAFT');
                wp_delete_post_revision($old_auto_save->ID);
                return true;
            }

            do_action('wp_creating_autosave', $new_auto_save);
            return wp_update_post($new_auto_save);
        }

        // _wp_put_post_revision() expects unescaped.
        $post_data = wp_unslash($post_data);

        // Otherwise create the new auto-save as a special post revision.
        $revision_id = _wp_put_post_revision($post_data, true);
        if ($revision_id) {
            update_post_meta($revision_id, 'WB4WP_DRAFT', true);
            return true;
        }

        return false;
    }

    /**
     * Copied from /wp-admin/includes/post.php
     *
     * @param null $post_data
     *
     * @return array|null
     */
    private static function _wp_get_allowed_post_data($post_data = null)
    {
        if (empty($post_data)) {
            $post_data = $_POST;
        }

        // Pass through errors.
        if (is_wp_error($post_data)) {
            return $post_data;
        }

        return array_diff_key($post_data, array_flip(array('meta_input', 'file', 'guid')));
    }

    /**
     * Get latest revision for a post
     *
     * @param $post_id
     *
     * @return int|WP_Post
     */
    private function get_latest_revision($post_id)
    {
        $posts = wp_get_post_revisions($post_id);

        usort($posts, function ( $a, $b ) {
            return strtotime($b->post_modified_gmt) - strtotime($a->post_modified_gmt);
        });
        $latest = reset($posts);

        return (!empty($latest) && !empty(get_post_meta($latest->post_parent, 'WB4WP_DRAFT', true))) ? $latest : null;
    }
}
